using System;
using Zcu.Mve.Core;

namespace Examples
{

	/// <summary>
  /// <p>Sample module. Source of one random scalar number.
  /// A double-precision floating point number greater than or equal
  /// to 0.0, and less than 1.0. Possibly you can define own output number.</p>
	/// </summary>
  [ModuleInfo("Milan Frank", "Sample module. Source of one random scalar number.",
      IconName = "numbersource.ico", Assemblied = "2004-06-13", Category = "Example")]
  [PortInfo("Output", "Random or user defined scalar number.")]
  public class NumberSource : Zcu.Mve.Core.Module
	{
    // CZE: Tyto hodnoty jsou interni aby k nim mohl pristupovat NumberSourceSetup.
		internal double val;
    internal Random random = new Random();
    internal ScalarNumber num = new ScalarNumber();
    internal bool dataSame = false;

    /// <summary>
    /// Nonparametric constructor. Create ports and set default values.
    /// </summary>
		public NumberSource()
		{
			AddOutPort("Output", typeof (Examples.ScalarNumber));
		}

    /// <summary>
    /// Main method of any module. Called on demand of output data.
    /// </summary>
    public override void Execute()
		{			
      if (random != null)
      {
        num.Val = random.NextDouble();
        SetOutput("Output", num, false);
      }
      else
      {
        num.Val = val;
        SetOutput("Output", num, dataSame);
        dataSame = true;
      }
 		}
	
    /// <summary>
    /// Configuration is saved to appropriate XML element in map file.
    /// </summary>
    /// <param name="config"></param>
		public override void WriteConfig(System.Xml.XmlElement config)
		{
			if (random != null)
				config.InnerText = "random";
			else
				config.InnerText = val.ToString();
		}

    /// <summary>
    /// Reads the configuration from XML map representation.
    /// </summary>
    /// <param name="config"></param>
		public override void ReadConfig(System.Xml.XmlElement config)
		{
			if (String.Compare(config.InnerText, "random", true) == 0)
				random = new Random();
			else
			{
				random = null;
				val = Int32.Parse(config.InnerText.Trim());
			}
		}

		/// <summary>
		/// You can choose from two modes of function of module:
		/// <ol>
    /// <li><p>Generating of random number:</p>
    /// <p><img src="Examples/numbersource1.jpg" /></p></li>
    /// <li><p>Generating of defined number (2.0 in this case):</p>
    /// <p><img src="Examples/numbersource2.jpg" /></p></li>
    /// </ol>
		/// </summary>
		/// <returns>User control.</returns>
    public override ModuleSetup InvokeSetup()
		{
      return new NumberSourceSetup(this);
		}
	

    // ZCE 13.4.2005 - deep copy bug fixed
    /// <summary>
    /// Deep copy of current module. Important for copy function
    /// of MapEditor. Module name is handled by MapEditor.
    /// </summary>
    /// <returns>Clone of this module.</returns>
    public override Module DeepCopy()
    {
      // There are several values that have to be cloned by DeepCopy of Module.
      NumberSource module = (NumberSource) base.DeepCopy ();

      // Copy of user vaules
      module.val = this.val;      
      if (this.random != null)
        module.random = new Random();
      else
        module.random = null;

      return module;
    }
  }		
}
