using System;
using Zcu.Mve.Core;

namespace Examples
{	
	/// <summary>
	/// Sample data class. It represents one scalar number
	/// (accuracy as double type in .NET). Can be shared by modules.<br></br>
	/// </summary>
	
  [DataObject("One scalar number (double)")]
  public class ScalarNumber : DataObject
	{
    /// <summary>
    /// Value carried by this data object.
    /// </summary>
		private double val;

    /// <summary>
    /// Constructor wit default setting.
    /// </summary>
    public ScalarNumber()
		{
			val = 0.0f;
		}

    /// <summary>
    /// Gets/Sets scalar value of this object.
    /// </summary>
    public double Val
		{
			get
			{
				return val;
			}
			set
			{
				val = value;
			}
		}

		/// <summary>
		/// Reads data from XML file into data object.
		/// </summary>
		/// <param name="xmlTextReader">Datovy tok pro cteni dat z Xml souboru.</param>
		public override void ReadData(System.Xml.XmlTextReader xmlTextReader)
		{
			this.Val = Double.Parse(xmlTextReader.ReadString());
		} // ReadData()
  

		/// <summary>
		/// Writes data object into XML file.
		/// </summary>
		/// <param name="xmlTextWriter">Datovy tok pro zapis dat do Xml souboru.</param>
		public override void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
		{
			xmlTextWriter.WriteString("" + this.Val);
		} // WriteData()


    /// <summary>
    /// User defined deep copy of this data object.
    /// </summary>
    /// <returns></returns>
		public override IDataObject DeepCopy()
		{
			ScalarNumber ret = new ScalarNumber();
			ret.Val = this.Val;
      
			return ret;
		}


	  /// <summary>
	  /// Report of the content of data structure. It's used by ConsolePrinter for example.
	  /// </summary>
	  /// <returns></returns>
    public override string ToString()
    {
      return this.val.ToString(Globals.nfi);
    }
  
    /// <summary>
    /// User defined check of consistence of this data object.
    /// Warnings can be written to Console as well as reasons of inconsistency.<br></br>
    /// </summary>
    /// <returns>True if it's consistent, false otherwise.</returns>
    public override bool CheckConsistence()
    {
      if (this.val == 0.0)
        Console.WriteLine("Warning! Number equals zero.");
      return true;
    }
  }
}
