using System;
using System.Diagnostics;
using Zcu.Mve.Core;

namespace Examples
{
	/// <summary>
	/// Sample module. It sums two input numbers.
	/// </summary>
  [Zcu.Mve.Core.ModuleInfo("Milan Frank", "Sample module. It sums two input numbers.",
     Category = "Example",
     IconName = "Sumator.ico")]
  
  [PortInfo("A", "Input number A")]
  [PortInfo("B", "Input number B")]
  [PortInfo("Sum", "Sum of numbers A + B")]  
  public class Sumator : Zcu.Mve.Core.Module
	{
		private ScalarNumber sum;

		/// <summary>
		/// Creates ports.
		/// </summary>
    public Sumator()
		{      
			AddInPort("A", typeof (Examples.ScalarNumber));
			AddInPort("B", typeof (Examples.ScalarNumber));
			AddOutPort("Sum", typeof (Examples.ScalarNumber));

			sum = new ScalarNumber();
    }

    /// <summary>
    /// Main method of any module. Called on demand of output data.
    /// </summary>
    public override void Execute()
    {
      // Get input data
      ScalarNumber a = (ScalarNumber) GetInput("A");
      ScalarNumber b = (ScalarNumber) GetInput("B");

      Console.WriteLine("Globals.useTrace = " + Globals.useTrace);
      // Check whether data are same as in previous Execute call.
      if (IsDataSame("A") && IsDataSame("B"))
      {      
        // In case all input data are same as in previous step we do not have to calculate them again.
        SetOutput("Sum", sum, true);
      }
      else
      {
        // Calculate sum.
        sum.Val = a.Val + b.Val;
        // Send data to outport.
        SetOutput("Sum", sum, false);
      }
    }
	}
}
